#include "itext.h"
#include <iconv.h>

char * itextStrStr(char *str0,char *str1)
{
        char *p,str[256];
        int i;

        i=0;
        for(p=str0;*p;p++){
                if(isspace(*p)) continue;
                str[i++] = *p;
        }
        str[i] = '\0';

        if((p = strstr(str,str1)) != NULL) return p;

        return NULL;
}
ftype_t itextCheckFileType_ex(char *fname)
{
	char *p;

	for(p=fname+(strlen(fname)-1);*p;p--){
		if(*p == '.'){
			if(	strcmp(p,".txt") == 0	|| 
				strcmp(p,".TXT") == 0 	|| 
				strcmp(p,".text") == 0	|| 
				strcmp(p,".TEXT") == 0	||
				strcmp(p,".Text") == 0	||
				strcmp(p,".Txt") == 0	){

			       		return F_TXT;

			}else if(	strcmp(p,".csv") == 0	||
					strcmp(p,".CSV") == 0	){
					
						return F_CSV;

			}else if(	strcmp(p,".zip") == 0	||
					strcmp(p,".ZIP") == 0	||
					strcmp(p,".Zip") == 0	){

						return F_ZIP;
			}else{
				return F_NIP;
			}
			break;
		}
	}

	return F_BIP;
}
fcode_t itextCheckKanjiCode(FILE *fp,char *fbuf)
{
	unsigned char *p,*pp;
	int flag;

	if(fp != NULL){
		unsigned char buf[1024];
		int i,n;

		fseek(fp,0L,SEEK_SET);
		i = 0;
		while((n = getc(fp)) > 0){
			buf[i++] = n;
			if(i > 1023) break;
		}
		pp = buf;
		fseek(fp,0L,SEEK_SET);

	}else if(fbuf != NULL){
		pp = fbuf;

	}else{
		return _U;
	}
	flag = 0;
	for(p=pp;*p;p++){
		if(*p == 0x1b && flag == 0)
		       	flag++;
		else if(*p == 0x24 && flag == 1)
		       	flag++;
		else if(*p == 0x26 && flag == 1)
		       	flag++;
		else if(*p == 0x40 && flag == 2)
			return _J;
		else if(*p == 0x42 && flag == 2)
			return _J;
		else if(*p == 0x28 && flag == 2)
			flag++;
		else if(*p == 0x44 && flag == 3)
			return _J;
		else
			flag = 0;
	}
	for(p=pp;*p;p++)
		if(*p > 0x80 && *p < 0xa0) return _S;
	for(p=pp;*p;p++)
		if(*p == 0xfd || *p == 0xfe) return _E;
	flag = 0;
	for(p=pp;*p;p++){
		if(*p & 0x80)
			flag = flag ? 0 : 1;
		if(flag)
			if(*p > 0xa1 && *p < 0xdf) return _E;
	}
	flag = 0;
	for(p=pp;*p;p++)
		if(*p > 0x20 && *p < 0x7f) flag++;
	if((flag * 100 / strlen(pp)) >= 80) return _U;

	return _S;
}
ftype_t itextCheckFileType(char *fname)
{
	FILE *fp;
	char buf[BUFSIZ];
	fcode_t code;
	ftype_t re = F_BIP;

	if((fp = fopen(fname,"r")) == NULL) return F_BIP;

	if(fgetc(fp) == 'P'){
		if(fgetc(fp) == 'K'){
			re = F_ZIP;
			goto _end;
		}
	}
	switch((code = itextCheckKanjiCode(fp,NULL))){
		case _J:case _E:case _S:
			re = F_TXT;
			goto _end;
		break;
	}
	re = itextCheckFileType_ex(fname);
_end:
	fclose(fp);
	return re;
}
iconv_t itextConvOpen(FILE *fp,char *buf)
{
        iconv_t con;

        switch(itextCheckKanjiCode(fp,buf)){
                case _J:con = iconv_open("EUC-JP","ISO2022JP");break;
                case _S:con = iconv_open("EUC-JP","SHIFT-JIS");break;
		case _E:con = NULL;break;
		default:con = NULL;break;
        }
        return con;
}
void itextConvClose(iconv_t con)
{
	if(con != NULL)
        	iconv_close(con);
}
int itextConvKanjiCode(iconv_t con,char *to,char *from)
{
        char *outbuf = NULL,*out = NULL;
        char *inbuf = NULL;
        int  in_len,out_len,from_len;

	if(from == NULL) return -1;

	from_len = strlen(from);
        in_len = from_len;
        out_len = from_len * 2;

        inbuf = from;
        if((outbuf = (char *)malloc(out_len)) == NULL){
                fprintf(stderr,"False itextConvertKanjiCode()(malloc())\n");
                return -1;
        }
        memset(outbuf,0,out_len);

        out = outbuf;
        iconv(con,&inbuf,&in_len,&outbuf,&out_len);
        strcpy(to,out);

        if(out != NULL) free(out);
        return 0;
}



