#include "itext.h"
#include "xjp.h"
#include <sys/stat.h>
/*#include <sys/time.h>*/

#if HAVE_LIBWKF
#include <wkf.h>
#endif

#define STR_HEAD_LINE 	"---------------------------"
#define STR_HEAD_LINE1 	"=========================="
#define MAX_HEAD_LINE 	30
#define EUC_BUFFER	2

extern char *XJP_MESSAGE[];

unsigned char SPACE = ' ';
unsigned char CHU_KEY_C1 = 0xa2;
unsigned char CHU_KEY_C2 = 0xb0;

static IBUF_T *ibuf_data_alloc()
{
	IBUF_T *it;

	if((it = (IBUF_T *)malloc(sizeof(IBUF_T))) == NULL){
		fprintf(stderr,"False ibuf_add()(malloc())\n");
		return NULL;
	}
	it->line = 0;
	it->i_head = NULL;
	it->i_tail = NULL;
	return it;
}
static void ibuf_data_add(IBUF_T **start,IBUF_T **end,IBUF_T *mg)
{
	if(*end){
		mg->next = NULL;
		mg->prev = *end;

		(*end)->next = mg;
		*end = mg;
	}else{
		mg->next = NULL;
		mg->prev = NULL;

		*end = mg;
		*start = mg;
	}
}
void ibuf_data_free(ICONTENT_T *book)
{
	IBUF_T *ib,*ibb;
	ITEXT_T *it,*itt;

	if(book->b_head == NULL) return;
	if(book->o_head != NULL) other_data_free(book->o_head);
	for(ib = book->b_head;ib!=NULL;ib=ibb){
		ibb = ib->next;
		itext_data_free(ib->i_head);
		free(ib);
	}
	book->b_head = NULL;
	book->b_tail = NULL;
	book->o_head = NULL;
	book->total_line = 0;

	return;
}
void itext_data_free(ITEXT_T *ihead)
{
	ITEXT_T *it,*itt;

	if(ihead == NULL) return;

	for(it=ihead;it!=NULL;it=itt){
		itt = it->next;
		if(it->chu_string != NULL) free(it->chu_string);
		if(it->rubi_head != NULL) itext_data_free(it->rubi_head);
		if(it->chu_head != NULL) itext_data_free(it->chu_head);

		free(it);
	}

	return ;
}
static ITEXT_T *itext_data_alloc()
{
	ITEXT_T *it;

	if((it = (ITEXT_T *)malloc(sizeof(ITEXT_T))) == NULL) return NULL;
	it->c1 = SPACE;
	it->c2 = SPACE;
	it->chu_string = NULL;
	it->off = 0;
	it->flag = 0;
	it->rubi_head = NULL;
	it->rubi_tail = NULL;
	it->chu_head = NULL;
	it->chu_tail = NULL;

	return it;
}
static void itext_data_add(ITEXT_T **start,ITEXT_T **end,ITEXT_T *mg)
{
	if(*end){
		mg->next = NULL;
		mg->prev = *end;

		(*end)->next = (ITEXT_T *)mg;
		*end = mg;
	}else{
		mg->next = NULL;
		mg->prev = NULL;

		*end = mg;
		*start = mg;
	}
}
static int _ibuf_add(	IBUF_T **head,
			IBUF_T **tail,
			ITEXT_T **ihead,
			ITEXT_T **itail,
			int i,
			int line	)
{
	IBUF_T *ib;

	if((ib = (IBUF_T *)ibuf_data_alloc()) == NULL) return -1;

	ib->line = line;
	ib->i_head = *ihead;
	ib->i_tail = *itail;
	ibuf_data_add(head,tail,ib);

	*ihead = NULL;
	*itail = NULL;

	return 0;
}
static int _check_cr(unsigned char *s,unsigned char *n)
{
	int i;
	unsigned char *p;

	i = 0;
	for(p=s;*p;p++){
		if(*p == '\r' || *p == '\n'){
			if(n != NULL) *p = *n;
			return i;
		}
		i++;
	}
	return -1;
}
int itextDataInsert(		ITEXT_T **ihead,
				ITEXT_T **itail,
				unsigned char c1,
				unsigned char c2,
				int ej,
				int chu,
				int small		)
{
	ITEXT_T *it;

	if((it = (ITEXT_T *)itext_data_alloc()) == NULL) return -1;
	it->c1 = c1;
	it->c2 = c2;

	if(ej)
		OnFlag(it->flag,_EJC);
	else
		OfFlag(it->flag,_EJC);

	if(chu)
		OnFlag(it->flag,_CHU);
	else
		OfFlag(it->flag,_CHU);

	if(small)
		OnFlag(it->flag,_SML);
	else
		OfFlag(it->flag,_SML);

	itext_data_add(ihead,itail,it);

	return 0;
}
static int _check_ruby(unsigned char c1,unsigned char c2,ITEXT_T *itail,int *rubi,int ruby,int start)
{
	static int rub;

	if(ruby == 0) return 0;

	if(start == 0) rub = -1;

	if(rub >= 0) rub++;
	if(c1 == 0xa1 && c2 == 0xc3){
		rub = 0;
		return 1;
	}
	if(c1 == 0xa1 && c2 == 0xd4 && *rubi == 0){
	        if(itail != NULL && rub >= 0){
			OnFlag(itail->flag,_RUB);
			SetFlag(itail->flag,_RUB);
		}
		rub = -1;
	        *rubi = 1;
		return 1;
	}
	if(c1 == 0xa1 && c2 == 0xd5 && *rubi == 1){
         	*rubi = 0;
		return 1;
	}
	return 0;
}
static int _check_chu(unsigned char c1,unsigned char c2,unsigned char *p,int *chu,int ruby)
{
	if(ruby == 0) return 0;

	if(c1 == 0xa1 && c2 == 0xce && *chu == 0){
	        if((*(p+1) == 0xa1 && *(p+2) == 0xf4) ||
                   (*(p+1) == 0xa1 && *(p+2) == 0xf6) ){
			*chu = 1;
			return 1;
		}
	}
        if(c1 == 0xa1 && c2 == 0xcf && *chu == 1){
		        *chu = 0;
			return 1;
	}
	return 0;
}
static int _check_header(unsigned char *buf_euc,int ruby,int start)
{
	static int line;
	static int head,head1;

	line++;
	if(start == 0){ line = 0; head = 0; head1 = 0;}
	if(line > MAX_HEAD_LINE || head1 == 1 || ruby == 0) return 0;

	if(strncmp(buf_euc,STR_HEAD_LINE,strlen(STR_HEAD_LINE)) == 0	){
		head = head ? 0 : 1;
		return 1;
	}
	if(strncmp(buf_euc,STR_HEAD_LINE1,strlen(STR_HEAD_LINE1)) == 0	){
		head1 = 1;
	}
	if(head == 0) return 0;

	return 1;
}
static int _buf_alloc(unsigned char **b,unsigned char **e,int size)
{
	if(*b != NULL){
		if((*b = (unsigned char *)realloc(*b,size)) == NULL){
			fprintf(stderr,"False _buf_alloc(1)(realloc())\n");
	       		return -1;
		}
	}else{
		if((*b = (unsigned char *)malloc(size)) == NULL){
			fprintf(stderr,"False _buf_alloc(1)(malloc())\n");
	       		return -1;
		}
		memset(*b,0,size);
	}

	if(*e != NULL){
		if((*e = (unsigned char *)realloc(*e,size * EUC_BUFFER)) == NULL){
			fprintf(stderr,"False _buf_alloc(2)(realloc())\n");
	       		return -1;
		}
	}else{
		if((*e = (unsigned char *)malloc(size * EUC_BUFFER)) == NULL){
			fprintf(stderr,"False _buf_alloc(2)(malloc())\n");
	       		return -1;
		}
		memset(*e,0,size * EUC_BUFFER);
	}
	return 0;
}
static void _buf_cat(unsigned char **b,unsigned char **e,unsigned char **u,int *size)
{
	int blen,elen;

	if(*b == NULL || *e == NULL || *u == NULL || *size <= 0){
		fprintf(stderr,"False buf_cat()\n");
		exit(1);
	}
	blen = strlen(*b); elen = strlen(*e);
	if(*size <= blen+elen){
		*size = blen + elen + 4;
		if((*b = (unsigned char *)realloc(*b,*size)) == NULL){
			fprintf(stderr,"False buf_cat()(reallo())\n");
			exit(1);
		}
	}
	sprintf(*b,"%s%s",*b,*e);

	if(_buf_alloc(e,u,*size) < 0)
	       	exit(1);

	strncpy(*e,*b,*size-1);

	if(*b != NULL) free(*b);
	*b = NULL;
	*size = 0;
	return;
}
FILE * _open_file(ICONTENT_T *book,char **zip,int *fsize)
{
	FILE *fp;
	struct stat sbuf;

	switch(itextCheckFileType(book->filename)){
		case F_TXT:
			if((fp = fopen(book->filename,"r")) == NULL){
				fprintf(stderr,"False open_file()(txt)\n");
				return NULL;
			}
		break;
		case F_ZIP:
			itextUnZipFile(book->filename,zip,&(book->o_head));
			if(*zip == NULL){
				fprintf(stderr,"False unzip_file()\n");
				return NULL;
			}
			if((fp = fopen(*zip,"r")) == NULL){
			       fprintf(stderr,"False open_file()(zip)\n");
		       	       return NULL;
			}
		break;
		default:
			       fprintf(stderr,"False open_file()(?)\n");
				return NULL;
		break;

	}
	fstat(fileno(fp),&sbuf);
	*fsize = sbuf.st_size;

	return fp;
}
int itextGetTextData(ICONTENT_T *book,int ruby_on,int buf_size)
{
	/*struct timeval tv0,tv1;*/
	iconv_t icon;

	FILE *fp;
	unsigned char *p,*tmp = NULL;
	unsigned char *buf = NULL;
	unsigned char *buf_euc = NULL;

	IBUF_T *ib;
	ITEXT_T *it,*rb,*chu;
	ITEXT_T *ihead = NULL;
	ITEXT_T *itail = NULL;

	unsigned char c1 = 0;
	unsigned char c2 = 0;

	int i,len,tmp_size = 0;
	int file_size = 0;
	int total_size = 0;
	int head_start = 0;
	int ruby_start = 0;
	int total_line = 1;
	char *zipfname = NULL;

	int RUBI = 0;
	int CHU = 0;
	int SMALL = 0;

	if((fp = _open_file(book,&zipfname,&file_size)) == NULL) return -1;

	if(_buf_alloc(&buf,&buf_euc,buf_size) < 0) return -1;

	icon = itextConvOpen(fp,NULL);
	xtext_usage(NULL,0);

	/*gettimeofday(&tv0,NULL);*/
	while(fgets(buf,buf_size,fp) != NULL){
		total_size += (len = strlen(buf));
		xtext_usage(XJP_MESSAGE[7],(double)total_size/file_size);

		if(*(buf+(len-1)) != '\r'&& *(buf+(len-1)) != '\n' && len > buf_size-2){

			if(_buf_alloc(&tmp,&buf_euc,tmp_size += buf_size * 2) < 0) return -1;
			sprintf(tmp,"%s%s",tmp,buf);
			continue;
		}
		if(tmp != NULL) _buf_cat(&tmp,&buf,&buf_euc,&tmp_size);
#if HAVE_LIBWKF
		wkfConvertKanjiCodeOfString(KC_UNKNOWN,buf,KC_EUC,buf_euc,buf_size * 2);
#else
		if(icon == NULL)
			strcpy(buf_euc,buf);
		else
			itextConvKanjiCode(icon,buf_euc,buf);
#endif
		if(_check_header(buf_euc,ruby_on,head_start++)) continue;
		if(CHU == 1){
			i = checkRubyString(i,&((book->b_tail)->i_head),&((book->b_tail)->i_tail),&SMALL);
		        CHU = 0;
		}
		if(_check_cr(buf_euc,"\0") == 0){
			if(SMALL == 1) SMALL = 0;
			_ibuf_add(&(book->b_head),&(book->b_tail),&ihead,&itail,0,total_line++); 
			continue;
		}
		if(book->titlename == NULL){
			book->titlename = (char *)malloc(strlen(buf_euc)+2);
			strcpy(book->titlename,buf_euc);
		}
		i=0;
		for(p=buf_euc;*p;p++){
			if(*p & 0x80){
				c1 = *p++;
				c2 = *p;

				if(_check_ruby(c1,c2,itail,&RUBI,ruby_on,ruby_start++)) 
					continue;
				if(_check_chu(c1,c2,p,&CHU,ruby_on)){
					if(CHU == 0) i = checkRubyString(i,&ihead,&itail,&SMALL);
					continue;
				}

				if(RUBI){
					itextDataInsert(&itail->rubi_head,&itail->rubi_tail,c1,c2,1,0,0);
				}else if(CHU){
					if(itail != NULL){
						itextDataInsert(&itail->chu_head,&itail->chu_tail,c1,c2,1,0,0);
					}else{
						i++;i++;
						itextDataInsert(&ihead,&itail,CHU_KEY_C1,CHU_KEY_C2,1,1,SMALL);
						itextDataInsert(&itail->chu_head,&itail->chu_tail,c1,c2,0,0,0);
					}
				}else{
					i++;i++;
					itextDataInsert(&ihead,&itail,c1,c2,1,0,SMALL);

				}
			}else{
				if(*p == '\t'){
					int xj;
					for(xj=0;xj<4;xj++){
						i++;
						c1 = SPACE;
						c2 = SPACE;

						itextDataInsert(&ihead,&itail,c1,c2,1,0,SMALL);
					}
				}else{
					i++;
					c1 = *p;
					c2 = SPACE;
					if(CHU)
						itextDataInsert(&itail->chu_head,&itail->chu_tail,c1,c2,0,0,SMALL);
					else
						itextDataInsert(&ihead,&itail,c1,c2,0,0,SMALL);
				}
			}
			
		}
		if(i > 0){
			if(RUBI == 1){falseRubyString(&ihead,&itail);RUBI=0;}
			_ibuf_add(&(book->b_head),&(book->b_tail),&ihead,&itail,i,total_line++); 
		}
	}
	/*gettimeofday(&tv1,NULL);
	printf("%d   %d\n",tv1.tv_sec - tv0.tv_sec,tv1.tv_usec - tv0.tv_usec);*/
	fclose(fp);
	itextConvClose(icon);
	free(buf);
	free(buf_euc);

	if(zipfname != NULL){
	       	unlink(zipfname);
		free(zipfname);
	}
	xtext_usage(NULL,0);

	return total_line;
}
